import Jcg.geometry.*;

/**
 * Class for rendering a surface triangle mesh (using Processing)
 * 
 * @author Luca Castelli Aleardi (INF555, 2012)
 *
 */
public class DrawPointSet {
	
	double scaleFactor=60; // scaling factor: useful for 3d rendering
	MeshViewer view; // Processing 3d frame (where meshes are rendered)
	public PointSet points1, points2; // input point clouds
	
	/**
	 * Create a surface mesh from an OFF file
	 */	
	public DrawPointSet(MeshViewer view) {
		this.view=view;
   	    this.points1=view.points1;
   	    this.points2=view.points2;
    	this.scaleFactor=this.computeScaleFactor();
	}
	
	/**
	 * Draw a point (as a small sphere)
	 */	
	public void drawPoint(Point_3 p) {
		float s=(float)this.scaleFactor;
		float x1=(float)p.getX().doubleValue()*s;
		float y1=(float)p.getY().doubleValue()*s;
		float z1=(float)p.getZ().doubleValue()*s;
		
		view.translate(x1, y1, z1);
		view.sphere(s/25f);
		view.translate(-x1, -y1, -z1);
	}

	/**
	 * Draw the entire point cloud
	 */
	public void draw(int type) {
		//this.drawAxis();
		
		view.noStroke();
		view.fill(0f, 0f, 250f);
		if(points1!=null)
			for(Point_3 p: this.points1.listOfPoints()) {
				this.drawPoint(p);
			}
		if(points2!=null) {
			view.fill(250f, 0f, 0f);
			for(Point_3 p: this.points2.listOfPoints()) {
				this.drawPoint(p);
			}
		}
		view.strokeWeight(1);
	}
	
	/**
	 * Compute the scale factor (depending on the max distance of the point set)
	 */
	public double computeScaleFactor() {
		if(this.points1==null || this.points1.size()<1)
			return 1;
		double maxDistance=0.;
		Point_3 origin=new Point_3(0., 0., 0.);
		for(Point_3 p: this.points1.listOfPoints()) {
			double distance=Math.sqrt(p.squareDistance(origin).doubleValue());
			maxDistance=Math.max(maxDistance, distance);
		}
		return Math.sqrt(3)/maxDistance*150;
	}
	
	/**
	 * Update the scale factor
	 */
	public void updateScaleFactor() {
		this.scaleFactor=this.computeScaleFactor();
	}
	
}
