import processing.core.*;

import Jcg.geometry.*;
import Jcg.polyhedron.*;

/**
 * A simple 3d viewer for visualizing surface meshes (based on Processing)
 * 
 * @author Luca Castelli Aleardi (INF555, 2012)
 *
 */
public class MeshViewer extends PApplet {

	PointSet points1, points2; // input point clouds
	RigidTransformation_3 icp;
	int renderType=0; // choice of type of rendering
	int renderModes=3; // number of rendering modes
	DrawPointSet drawPointSet;
	
	Rotation_3 rotation=new Rotation_3(); // identity rotation by default
	Translation_3 translation=new Translation_3(new Vector_3(0., 0., 0.)); // no translation
	String filename1, filename2;// input point clouds
	
	public void setup() {
		  size(800,600,P3D);
		  ArcBall arcball = new ArcBall(this);
		  filename1="OFF/tore.off";
		  filename2="OFF/tore_rotation1.off";
	  
		  this.points1=new PointSet(filename1);
		  this.points2=new PointSet(filename2);
		  
		  this.drawPointSet=new DrawPointSet(this);
		  this.icp=new IterativeClosestPoint();
	}
		 
		public void draw() {
		  background(0);
		  //this.lights();
		  directionalLight(101, 204, 255, -1, 0, 0);
		  directionalLight(51, 102, 126, 0, -1, 0);
		  directionalLight(51, 102, 126, 0, 0, -1);
		  directionalLight(102, 50, 126, 1, 0, 0);
		  directionalLight(51, 50, 102, 0, 1, 0);
		  directionalLight(51, 50, 102, 0, 0, 1);
		 
		  translate(width/2.f,height/2.f,-1*height/2.f);
		  this.strokeWeight(1);
		  stroke(150,150,150);
		  
		  this.drawPointSet.draw(renderType);
		}
		
		public void keyPressed(){
			  switch(key) {
			    case('i'):case('I'): this.computeICP(); break;
			    case('p'):case('P'): this.points2.permute(); this.drawPointSet=new DrawPointSet(this); break;
			    case('r'):this.renderType=(this.renderType+1)%this.renderModes; break;
			    case('s'):case('S'): this.savePointCloud(); break;
			    case('t'):case('T'): this.transformPointCloud(); this.drawPointSet=new DrawPointSet(this); break;
			  }
		}
		
		public void computeICP() {
			this.rotation=this.icp.getRotation(points1, points2);
			this.translation=this.icp.getTranslation(points1, points2);
		}
		
		public void savePointCloud() {
			if(points2!=null)
				points2.toFile("output.off");
		}

		public void transformPointCloud() {
			//throw new Error("To be completed: TD6");
		}

		/**
		 * For running the PApplet as Java application
		 */
		public static void main(String args[]) {
			//PApplet pa=new MeshViewer();
			//pa.setSize(400, 400);
			PApplet.main(new String[] { "MeshViewer" });
		}

}
